@@ -97,6 +97,7 @@
           ((symbolp value) (intern (string value) (symbol-package symbol)))
           (t (nesl-error "~s is an invalid form" symbol)))))
 
+#|
 (defun convert-vexpt-list (list)
   (cond ((null list) (cons nil nil))
 	((eql (car list) 'v.)
@@ -106,10 +107,28 @@
 	 (let ((rest (convert-vexpt-list (cdr list)))
 	       (this (if (and (symbolp (car list)) (vprefixp (car list)))
 			 (cons nil (vpostfix (car list)))
-		       (let ((sym (gensym "ARG-")))
+		       (let ((sym (gensym "x")))
 			 (cons (list (list sym (car list))) sym)))))	
 	   (cons (append (car this) (car rest))
 		 (cons (cdr this) (cdr rest)))))))
+|#
+
+(defun convert-vexpt-list (list)
+  (cond ((null list) (cons nil nil))
+	((eql (car list) 'v.)
+	 (let ((rest (convert-vexpt-list (cddr list))))
+	   (cons (car rest) (cons (cadr list) (cdr rest)))))
+	(t
+	 (let ((rest (convert-vexpt-list (cdr list)))
+	       (this (if (symbolp (car list))
+			 (if (vprefixp (car list))
+			     (cons nil (vpostfix (car list)))
+			   (cons (list (list (car list) (car list))) 
+				 (car list)))
+		       (let ((sym (gensym "x")))
+			 (cons (list (list sym (car list))) sym)))))
+	   (cons (append (car this) (car rest))
+		 (cons (cdr this) (cdr rest)))))))
 
 (defun convert-vexp (exp)
   (if (and (symbolp (car exp)) (vprefixp (car exp)))
@@ -135,23 +154,26 @@
       (nesl-error 
        "In an over expression, there must be at least one binding.~%~s"
        exp))
-     `(with (((nesl::vector segdes ,(first (first bindings)))
+     `(with (((nesl::vector (nesl::pair segdes ,(first (first bindings))))
 	      ,(conv-exp (second (first bindings)) pflag env))
 	     ,@(mapcar #'(lambda (a)
-			   `((nesl::vector segdes ,(first a))
+			   `((nesl::vector (nesl::pair segdes ,(first a)))
 			     ,(conv-exp (second a) pflag env)))
 		       (cdr bindings))
 	     ,@(mapcar (if pflag
 			   #'(lambda (a) 
-			       `(,a (neslp::prim-dist ,a segdes seg_len)))
-			 #'(lambda (a) `(,a (nesl::prim-dist ,a segdes))))
+			       `(,a (neslp::prim-dist 
+				     (nesl::pair (nesl::pair ,a segdes)
+						 seg_len))))
+			 #'(lambda (a) `(,a (nesl::prim-dist (nesl::pair ,a segdes)))))
 		       free-body)
 	     (seg_len 
 	      ,(if pflag 
 		   '(nesl::make-segdes 
-		     (nesl::prim-+-reduce (nesl::len segdes) seg_len))
+		     (nesl::prim-+-reduce 
+		      (nesl::pair (nesl::len segdes) seg_len)))
 		 'segdes)))
-	 (nesl::vector segdes ,body))))
+	 (nesl::vector (nesl::pair segdes ,body)))))
 
 (defun conv-bindings (bindings pflag env)
   (cond ((null bindings) nil)
@@ -168,7 +190,7 @@
 ;; occurences.
 (defun conv-constant (constant pflag env)
   (declare (ignore pflag))
-  `(nesl::prim-dist ,(conv-exp constant nil env) seg_len))
+  `(nesl::prim-dist (nesl::pair ,(conv-exp constant nil env) seg_len)))
   
 (defun conv-list (list pflag env)
   (cond ((null list) nil)
@@ -180,17 +202,16 @@
       (nesl-error "~a is an invalid function name." name)
     (intern (string name) 'neslp)))
 
-(defun add-parg (exp pflag)
-  (if pflag (append exp (list 'seg_len)) exp))
-
 (defun conv-func (expression pflag env)
   (let ((funname (if pflag (pname (car expression)) (car expression)))
 	(args (conv-list (cdr expression) pflag env)))
-    (add-parg (cons funname args) pflag)))
+    (if pflag
+	`(,funname (nesl::pair ,(first args) seg_len))
+      `(,funname ,@args))))
 
 (defun do-thing (a)
-  `((nesl::vector junk-segdes ,a) 
-    (nesl::pack (nesl::vector seg_len ,a) vcond)))
+  `((nesl::vector (nesl::pair junk-segdes ,a))
+    (nesl::oldpack (nesl::pair (nesl::vector (nesl::pair seg_len ,a)) vcond))))
 
 (defun conv-if (expression pflag env)
   (if (not pflag)
@@ -201,24 +222,27 @@
 	  (then-vars (remove-duplicates (ext-exp (third expression) nil)))
 	  (else-vars (remove-duplicates (ext-exp (fourth expression) nil))))
       `(with ((cond ,cond)
-	      (vcond (nesl::vector seg_len cond))
+	      (vcond (nesl::vector (nesl::pair seg_len cond)))
 	      (total-count (nesl::len seg_len))
 	      (true-count (nesl::count vcond))
-	      (false-count (nesl::- total-count true-count)))
-         (if (nesl::= true-count total-count)
+	      (false-count (nesl::- (nesl::pair total-count true-count))))
+         (if (nesl::= (nesl::pair true-count total-count))
 	     ,then
-	   (if (nesl::= false-count total-count)
+	   (if (nesl::= (nesl::pair false-count total-count))
 	       ,else
-	     (with (((nesl::vector if-segdes if-values)
+	     (with (((nesl::vector (nesl::pair if-segdes if-values))
 		     (nesl::flag-merge 
-		      vcond
-		      (with ((vcond (nesl::vector seg_len (nesl::not cond)))
-			     ,@(mapcar #'do-thing else-vars)
-			     (seg_len (nesl::make-segdes false-count)))
-			(nesl::vector seg_len ,else))
-		      (with (,@(mapcar #'do-thing then-vars)
-			       (seg_len (nesl::make-segdes true-count)))
-			(nesl::vector seg_len ,then)))))
+		      (nesl::pair 
+		       vcond
+		       (nesl::pair
+			(with ((vcond (nesl::vector 
+				       (nesl::pair seg_len (nesl::not cond))))
+			       ,@(mapcar #'do-thing else-vars)
+			       (seg_len (nesl::make-segdes false-count)))
+			  (nesl::vector (nesl::pair seg_len ,else)))
+			(with (,@(mapcar #'do-thing then-vars)
+				 (seg_len (nesl::make-segdes true-count)))
+			  (nesl::vector (nesl::pair seg_len ,then))))))))
 	       if-values)))))))
 
 (defun conv-exp (expression pflag env)
@@ -232,6 +256,8 @@
 	 (let ((nexp (convert-vexp expression)))
 	   (cond ((eql (car nexp) 'if)
 		  (conv-if nexp pflag env))
+		 ((eql (car nexp) 'nesl::pair)
+		  `(nesl::pair ,@(conv-list (cdr nexp) pflag env)))
 		 ((eql (car nexp) 'nesl::nif)
 		  (nesl-error "NIF is outdated, please use IF."))
 		 ((eql (car nexp) 'nesl::sif)
@@ -254,19 +280,25 @@
 	   (free-variables (remove-duplicates 
 			    (ext-exp body flat-variables)))
 	   (converted-body (conv-exp body t nil)))
-      `(with (,@(mapcar #'(lambda (a) `(,a (nesl::prim-dist ,a seg_len)))
+      `(with (,@(mapcar #'(lambda (a) `(,a (nesl::prim-dist 
+					    (nesl::pair ,a seg_len))))
 			free-variables))
 	 ,converted-body))))
 
 (defun conv-type (type pflag)
   (if pflag
-      (append type (list 'nesl::segdes))
+      `(,(first type) (nesl::pair ,(second type) nesl::segdes))
     type))
 
-(defun conv-types (types pflag)
-  (mapcar #'(lambda (a) (conv-type a pflag)) types))
+;;(defun conv-types (types pflag)
+;;  (mapcar #'(lambda (a) (conv-type a pflag)) types))
+
+;;(defun conv-names (names pflag)
+;;  (if pflag
+;;      (append (cons (pname (car names)) (cdr names)) (list 'seg_len))
+;;    names))
 
 (defun conv-names (names pflag)
   (if pflag
-      (append (cons (pname (car names)) (cdr names)) (list 'seg_len))
+      (list (pname (first names)) `(nesl::pair ,(second names) seg_len))
     names))
