@@ -57,7 +57,7 @@
 	       nesl::set))
 
 (defparameter *top-level-commands* 
-  '(progn nesl::use-machine nesl::cm-finger nesl::set-argcheck
+  '(progn nesl::use-machine nesl::cm-finger nesl::set_argcheck nesl::qstat
      nesl::use_machine nesl::cmfinger nesl::set_memory_size nesl::list_machines
      nesl::configuration nesl::set-memsize nesl::set-username
      nesl::list-machines))
@@ -98,28 +98,16 @@
 ;;; HELP FACILITIES
 ;;;;;;;;;;;;;
 
-(defun pretty-function-interface (names infix)
-  (let ((len (length names)))
-    (cond ((= len 1) (format nil "~a()" (first names)))
-	  ((and infix (= len 2))
-	   (format nil "~a~a" infix (second names)))
-	  ((and infix (= len 3))
-	   (format nil "~a ~a ~a" (second names) infix (third names)))
-	  (t
-	   (format nil "~a(~a~{,~a~})" 
-		   (first names) (second names) (cddr names))))))
-
 (defun describe-nesl (funname definitions)
   (let ((fundef (get-fundef funname definitions)))
     (cond ((not funname)
 	   (dolist (fun *doclist*)
-	     (if (symbolp fun)
-	       (let ((fundef (get-fundef fun definitions)))
+	     (if (and (symbolp fun) (get-fundef fun definitions))
+	       (let* ((fundef (get-fundef fun definitions))
+		      (interface (pprint-nesl-string (fundef-names fundef))))
 		 (if (fundef-shortdoc fundef)
-		     (format t "~%~25a ~a" (fundef-names fundef)
-			     (fundef-shortdoc fundef))
-		   (format t "~%~a" (fundef-names fundef))))
-	       (format t "~%"))))
+		     (format t "~25a ~a" interface (fundef-shortdoc fundef))
+		   (format t "~a" interface))))))
 	  ((member funname *special-forms*)
 	   (format t "~%~a is a special form; please see the NESL manual." 
 		   funname))
@@ -129,10 +117,9 @@
 	  (fundef
 	   (let ((interface (fundef-names fundef))
 		 (type (fundef-type fundef))
-		 (infix (fundef-infix fundef))
 		 (documentation (fundef-documentation fundef)))
-	     (format t "~%INTERFACE:~% ~a~%~%" 
-		     (pretty-function-interface interface infix))
+	     (format t "~%INTERFACE:~% ~a~%~%"  
+		     (pprint-nesl-string interface))
 	     (when type (format t "TYPE:~% ~a~%~%" 
 				(pretty-type-func type)))
 	     (when documentation 
@@ -183,7 +170,9 @@ Commands for running VCODE on remote machines:~%  ~
    CONFIGURATION();      -- List the properties of the current configuration.~%  ~
    USE_MACHINE(config);  -- Use the machine with configuration CONFIG.~%  ~
    LIST_MACHINES();      -- List the available configurations.~%  ~
-   SET_MEMORY_SIZE(n);   -- Set the memory size of the current configuration."
+   SET_MEMORY_SIZE(n);   -- Set the memory size of the current configuration.
+   name &= exp [,MEM := exp] [,MAXTIME := exp];
+                         -- Executes exp in the background~%~%"
 	  ))
 
 (defun nesl-list-bugs ()
@@ -216,7 +205,7 @@ Commands for running VCODE on remote machines:~%  ~
 	       (t (cons nil filename)))))
     (values (car type-name) (cdr type-name))))
 
-(defun load-nesl (filename &key (verbose t) (print t))
+(defun load-nesl (filename &key (verbose t) (print nil))
   (when (not (stringp filename))
     (nesl-error "The second argument to LOAD must be a character string."))
   (multiple-value-bind (type name) (get-filename-and-type filename)
@@ -242,6 +231,21 @@ Commands for running VCODE on remote machines:~%  ~
 		 (declare (special *cnesl-syntax*))
 		 (nesl-loop loadstr :interactive nil :print print))))))))
 
+(defun trans-nesl-to-cnesl (filename)
+  (let ((infile (concatenate 'string filename ".nesl"))
+	(outfile (concatenate 'string filename ".cnesl"))
+	(*cnesl-syntax* nil))
+    (when (probe-file outfile) (delete-file outfile))
+    (with-open-file (instr infile :direction :input)
+      (with-open-file (outstr outfile :direction :output)
+	(loop
+	 (let ((readval (nesl-read instr nil)))
+	   (if (nesl-exit? readval)
+	       (return)
+	     (progn
+	       (pprint-nesl readval outstr)
+	       (format outstr " $~%")))))))))
+
 ;;;;;;;;;;;;;;;;
 ;;; EVALUATING A NESL FORM
 ;;;;;;;;;;;;;;;;
@@ -258,32 +262,47 @@ Commands for running VCODE on remote machines:~%  ~
 		   result)
     form))
 
-(defun eval-nesl (form definitions)
+(defun eval-nesl (form definitions &optional background)
   (cond ((nesl-constant-p form)
 	 (values (coerce-nesl-constant form)
 		 (nesl-constant-p form)))
 	((listp form)
-	 (typecheck-op '(main) nil form definitions)
-	 (when *verbose* (format t "~%Compiling VCODE...."))
-	 (let* ((rtype-code (trans-op 
-			     nil nil 
-			     (trans-phase1 nil (verbose-wrapper form) nil)
-			     definitions))
-		(code (cdr rtype-code))
-		(result-type (car rtype-code)))
-	   (add-cached-code (get-fundef 'main definitions)
-			    nil code definitions)
-	   (values 
-	    (group-data 
-	     (run-vcode 'main (flatten-type result-type definitions) 
-			definitions)
-	     result-type definitions)
-	    result-type)))
+	 (let ((rtype (caar (typecheck-op '(main) nil form definitions)))
+	       (*print-pretty* nil))
+	   (declare (special *print-pretty*))
+	   (when *verbose* (format t "~%Compiling VCODE...."))
+	   (let* ((rtype-code (trans-op 
+			       nil nil 
+			       (trans-phase1 nil (verbose-wrapper form) nil)
+			       definitions))
+		  (code (cdr rtype-code))
+		  (result-type (car rtype-code)))
+	     (add-cached-code (get-fundef 'main definitions)
+			      nil code definitions)
+	     (values 
+	      (if background
+		  (run-vcode-background 'main definitions background)
+		(group-data 
+		 (run-vcode 'main (flatten-type result-type definitions)
+			    definitions)
+		 rtype definitions))
+	      rtype))))
 	((symbolp form)
 	 (let ((val-type (get-variable form definitions)))
-	   (if val-type 
-	       (values (car val-type) (cdr val-type))
-	     (nesl-error "Variable ~a is undefined." form))))
+	   (cond ((and (listp (car val-type))
+		       (eql `background (caar val-type)))
+		  (let* ((out-err-files (second (car val-type)))
+			 (type (cdr val-type))
+			 (flattype (flatten-type type definitions))
+			 (res (get-background-result out-err-files flattype))
+			 (val (group-data res type definitions)))
+		    (cond (res
+			   (add-variable form val type definitions)
+			   (values val type))
+			  (t (nesl-error "Variable waiting for result.")))))
+		 (val-type 
+		  (values (car val-type) (cdr val-type)))
+		 (t (nesl-error "Variable ~a is undefined." form)))))
 	(t (nesl-error "Invalid form: ~a" form))))	 
 
 ;;;;;;;;;;;;;;;;;;;
@@ -298,6 +317,18 @@ Commands for running VCODE on remote machines:~%  ~
 	     (apply (car form) (mapcar 
 				#'(lambda (form) (eval-nesl form definitions))
 				(cdr form))))
+	    ((eql (car form) 'noop)
+	     (values nil nil))
+	    ((eql (car form) 'nesl::ptrans)
+	     (pprint-nesl
+	      (conv-body nil (strip-exp (second form) definitions) nil)))
+	    ((eql (car form) 'nesl::ptransp)
+	     (pprint-nesl
+	      (conv-body nil (strip-exp (second form) definitions) t)))
+	    ((eql (car form) 'nesl::pprint)
+	     (pprint-nesl  (second form)))
+	    ((eql (car form) 'nesl::cnesl_trans)
+	     (trans-nesl-to-cnesl (second form)))
 	    ((eql (car form) 'nesl::set)
 	     (when (not (eql (length form) 3))
 	       (nesl-error "The syntax for SET is: (SET Ident exp)."))
@@ -354,6 +385,16 @@ Commands for running VCODE on remote machines:~%  ~
 	     (if (setq *debug* (not *debug*))
 		 (format t "~%Debug: On")
 	       (format t "~%Debug: Off")))
+	    ((eql (car form) 'background)
+	     (let* ((var (second form))
+		    (exp (car-pair (third form)))
+		    (mem-time-qname (flatten-pair (cdr-pair (third form)))))
+	       (check-valid-varname var definitions)
+	       (multiple-value-bind (out-err-files type)
+		   (eval-nesl exp definitions mem-time-qname)
+		 (add-variable var `(background ,out-err-files) 
+			       type definitions)
+		 (values (coerce-nesl-constant "stub") type))))
 	    (t (eval-nesl form definitions)))
       (eval-nesl form definitions)))
 
@@ -361,11 +402,13 @@ Commands for running VCODE on remote machines:~%  ~
 ;;; THE NESL READ/EVAL/PRINT LOOP
 ;;;;;;;;;;;;;
 
-(defun nesl-read (in-stream)
+(defun nesl-read (in-stream interactive)
   (if *CNESL-SYNTAX*
       ;; this is a real hack since cgolread doesn't work
-      (or (cgolread in-stream nil :exit) :exit)
-    (read in-stream nil :exit)))
+      (prog1 (or (cgolread in-stream nil :exit) :exit)
+	(when interactive (read-line *standard-input* nil)))
+    (let ((result (nesl-read-toplevel (read in-stream nil :exit))))
+      result)))
 
 (defun pretty-print-function (names)
   (if *cnesl-syntax*
@@ -379,8 +422,12 @@ Commands for running VCODE on remote machines:~%  ~
   (if type
       (if (and (listp type) (eql (car type) 'function))
 	  (format t "~%~a : ~a" 
-		  (pretty-print-function value) (pretty-type-func (cdr type)))
-	(format t "~%~s : ~a" (nest-constant value) (pretty-type type)))
+		  (pretty-type value) (pretty-type-func (cdr type)))
+	(if *cnesl-syntax*
+	    (format t "~%~a : ~a" 
+		    (cnesl-print-data (nest-constant value) t)
+		    (pretty-type type))
+	  (format t "~%~s : ~a" (nest-constant value) (pretty-type type))))
     nil))
 
 (defun nesl-exit? (readval)
@@ -397,7 +444,7 @@ Commands for running VCODE on remote machines:~%  ~
     (loop
      (when interactive (format t "~%<Nesl> ") (force-output))
      (catch 'nesl-error
-       (let ((readval (nesl-read in-stream)))
+       (let ((readval (nesl-read in-stream interactive)))
 	 (if (nesl-exit? readval)
 	     (return)
 	   (multiple-value-bind (value type) 
@@ -405,6 +452,7 @@ Commands for running VCODE on remote machines:~%  ~
 	     (when print (nesl-print value type)))))))))
 
 (defun nesl ()
-  (nesl-loop t :interactive t :print t))
+  (format t "~%; NESL version ~a (~a)~%" *nesl-version* *nesl-date*)
+  (nesl-loop *standard-input* :interactive t :print t))
 
 (defun user::nesl () (nesl))
