@@ -42,6 +42,8 @@
   (or (eql type 'nesl::any)
       (subtypes type definitions)))
 
+(defparameter *allow-type-vars* nil)
+
 (defun parse-type-exp (type tstack sum-func prim-func list-func 
 			    zero definitions)
   (cond ((null type) zero)
@@ -68,7 +70,9 @@
 		(car type)
 		(parse-type-list (cdr (car type-dec)) nstack sum-func 
 				 prim-func list-func zero definitions))))))
-	(t (nesl-error "No base-type definition for ~a." type))))
+	(t (if *allow-type-vars*
+	       (funcall prim-func type)
+	     (nesl-error "No base-type definition for ~a." type)))))
 
 (defun parse-type-list (type tstack sum-func prim-func list-func zero defs)
   (if type
@@ -83,18 +87,24 @@
   (parse-type-exp type nil #'append #'(lambda (a) (list a))
 		  #'(lambda (a b) (declare (ignore a)) b) nil definitions))
 
-(defun l-from-type (type definitions)
-  (parse-type-exp type nil #'+ #'(lambda (a) (declare (ignore a)) 1) 
-		  #'(lambda (a b) (declare (ignore a)) b) 0 definitions))
+;;(defun l-from-type (type definitions)
+;;  (parse-type-exp type nil #'+ #'(lambda (a) (declare (ignore a)) 1) 
+;;		  #'(lambda (a b) (declare (ignore a)) b) 0 definitions))
 
-(defun l-from-type-list (type definitions)
-  (parse-type-list type nil #'+ #'(lambda (a) (declare (ignore a)) 1)
-		   #'(lambda (a b) (declare (ignore a)) b) 0 definitions))
+;;(defun l-from-type-list (type definitions)
+;;  (parse-type-list type nil #'+ #'(lambda (a) (declare (ignore a)) 1)
+;;		   #'(lambda (a b) (declare (ignore a)) b) 0 definitions))
 
 (defun expand-type (type definitions)
   (parse-type-exp type nil #'cons #'(lambda (a) a) 
 		  #'cons nil definitions))
 
+(defun expand-type-allow (type definitions)
+  (let ((*allow-type-vars* t))
+    (declare (special *allow-type-vars*))
+    (parse-type-exp type nil #'cons #'(lambda (a) a) 
+		    #'cons nil definitions)))
+
 (defun andfunc (a b) (or a b))
 
 (defun check-type-list (full-type definitions)
@@ -171,20 +181,25 @@
 		 (mapcar #'pretty-type-nesl (cdr type))))
 	(t (nesl-error "Invalid type ~a" type))))
 
-(defun pretty-type-cnesl (type)
+(defun pretty-type-cnesl (type &optional parenth?)
   (cond ((atom type)
 	 (string type))
 	((listp type)
-	 (let ((subtype (mapcar #'pretty-type-cnesl (cdr type))))
-	   (cond ((eql (first type) 'nesl::vector)
-		  (format nil "[~a]" (car subtype)))
-		 ((eql (first type) 'function)
-		  (format nil "~a~{,~a~} -> ~a" 
-			  (second subtype) (cddr subtype) (car subtype)))
-		 ((eql (first type) 'nesl::pair)
-		  (format nil "(~a,~a)" (first subtype) (second subtype)))
-		 (t (format nil "~a(~a~{,~a~})" 
-			    (car type) (car subtype) (cdr subtype))))))
+	 (cond ((eql (first type) 'nesl::vector)
+		(format nil "[~a]" (pretty-type-cnesl (second type) t)))
+	       ((eql (first type) 'function)
+		(format nil "~a -> ~a" 
+			(pretty-type-cnesl (third type))
+			(pretty-type-cnesl (second type))))
+	       ((eql (first type) 'nesl::pair)
+		(format nil (if parenth? "(~a,~a)" "~a,~a")
+			(pretty-type-cnesl (second type) t)
+			(pretty-type-cnesl (third type))))
+	       ((eql (length type) 1)
+		(format nil "~a" (car type)))
+	       (t (format nil "~a(~a)" 
+			  (car type) 
+			  (pretty-type-cnesl (second type))))))
 	(t (nesl-error "Invalid type ~a" type))))
 
 (defun pretty-type (type)
@@ -221,3 +236,22 @@
   (format nil "~a ~a" 
 	  (pretty-type (cons 'function (car type)))
 	  (pretty-type-type-binds (cdr type))))
+
+
+;;; ******************
+;;; THe following is a special case of code given above.
+;;; It was inserted for efficiency reasons
+;;; ****************
+
+(defun l-from-type (type definitions)
+  (cond ((null type) 0)
+	((and (atom type) (primitive-type? type definitions)) 1)
+	((listp type)
+	 (cond ((eql (car type) 'nesl::pair)
+		(+ (l-from-type (second type) definitions)
+		   (l-from-type (third type) definitions)))
+	       ((eql (car type) 'nesl::vector)
+		(+ 1 (l-from-type (second type) definitions)))
+	       (t (nesl-error "Bad type ~a." type))))
+	(t (nesl-error "No base-type definition for ~a." type))))
+
