@@ -47,6 +47,7 @@ prog_entry_t	program[PROG_SIZE_MAX];
 int 		next_prog_num = 0;
 
 static void main_loop();
+static void initialize();
 static int init_pc();
 
 int lex_trace = 0;
@@ -57,7 +58,7 @@ int runtime_trace = 0;
 int program_dump = 0;
 int link_trace = 0;
 int timer = 0;
-int vstack_size_init = 100000;
+unsigned int vstack_size_init = 100000;
 int garbage_notify = 1;
 int check_args = 0;
 int debug_flag = 0;
@@ -102,6 +103,10 @@ char **argv;
     char *filename = NULL;
     FILE *file;
 
+#ifdef CM5
+    CVL_init();			/* set up file modes before anything else */
+    printf("done with CVL init\n");
+#endif
     while ((c = getopt(argc, argv, "cd:gm:r:t")) != EOF) {
 	switch (c) {
 	    case 'c':
@@ -134,8 +139,7 @@ char **argv;
 		timer = 1;
 		break;
 	    case 'm':	/* vstack size */
-		if (! sscanf(optarg,"%d", &vstack_size_init)
-		    || vstack_size_init <= 0)
+		if (! sscanf(optarg,"%u", &vstack_size_init))
 		    usage();
 		break;
 	    case 'g': 
@@ -171,11 +175,29 @@ char **argv;
     if (program_dump)
 	show_program();
 
+    initialize();
     main_loop();
     (void) fclose(file);
     exit(0);
 }
 
+static void initialize()
+{
+#ifdef CM2
+#ifdef DEBUG
+    CVL_init(1);			/* safety mode on -- slow */
+#else
+    CVL_init(0);			/* safety mode off -- fast'n'deadly */
+#endif DEBUG
+#endif CM2
+    assert(ok_vcode_table());		/* This doesn't do anything yet */
+    stack_init();
+    vstack_init(vstack_size_init);
+    rtstack_init();
+    rtstack_push(TOP_LEVEL);	/* End of Program Marker */
+    const_install();		/* put constants into memory */
+}
+
 /* main interpreter loop */
 static void main_loop()
 {
@@ -183,24 +205,6 @@ static void main_loop()
     cvl_timer_t begin_time, 		/* used by -t flag */
 	 	start_time, end_time;	/* used by START_TIMER / STOP_TIMER */
 
-#ifdef CM2
-#ifdef DEBUG
-    CVL_init(1);			/* safety mode on -- slow */
-#else
-    CVL_init(0);			/* safety mode off -- fast'n'deadly */
-#endif DEBUG
-#endif CM2
-#ifdef CM5
-    CVL_init();
-#endif
-
-    assert(ok_vcode_table());		/* This doesn't do anything yet */
-    stack_init();
-    vstack_init(vstack_size_init);
-    rtstack_init();
-    rtstack_push(TOP_LEVEL);	/* End of Program Marker */
-    const_install();		/* put constants into memory */
-
     pc = init_pc();		/* initialize program counter */
 
     if (timer)
@@ -266,6 +270,9 @@ static void main_loop()
 		case FREAD:
 		    do_fread(instruction);
 		    break;
+		case FREAD_CHAR:
+		    do_fread_char(instruction);
+		    break;
 		case FWRITE:
 		    do_fwrite(instruction);
 		    break;
